@@ -1,7 +1,8 @@
 from enum import unique, Enum
-from typing import Union, Callable, Coroutine, Optional
+from typing import Union, Callable, Coroutine, Optional, Tuple, List, Any
 
 from ._string_utils import assert_string_is_not_empty
+from .client_details import ClientDetails
 
 
 class KustoConnectionStringBuilder:
@@ -11,6 +12,8 @@ class KustoConnectionStringBuilder:
         https://github.com/Azure/azure-kusto-python/blob/master/azure-kusto-data/tests/sample.py
     """
 
+    kcsb_invalid_item_error = "%s is not supported as an item in KustoConnectionStringBuilder"
+
     @unique
     class ValidKeywords(Enum):
         """
@@ -120,14 +123,23 @@ class KustoConnectionStringBuilder:
         self._internal_dict = {}
         self._token_provider = None
         self._async_token_provider = None
+        self.is_token_credential_auth = False
+        self.credential: Optional[Any] = None
+        self.credential_from_login_endpoint: Optional[Any] = None
         if connection_string is not None and "=" not in connection_string.partition(";")[0]:
             connection_string = "Data Source=" + connection_string
 
+        self._application_for_tracing: Optional[str] = None
+        self._user_for_tracing: Optional[str] = None
+
         self[self.ValidKeywords.authority_id] = "organizations"
 
         for kvp_string in connection_string.split(";"):
             key, _, value = kvp_string.partition("=")
-            keyword = self.ValidKeywords.parse(key)
+            try:
+                keyword = self.ValidKeywords.parse(key)
+            except KeyError:
+                raise KeyError(self.kcsb_invalid_item_error % key)
             value_stripped = value.strip()
             if keyword.is_str_type():
                 self[keyword] = value_stripped.rstrip("/")
@@ -143,7 +155,7 @@ class KustoConnectionStringBuilder:
         try:
             keyword = key if isinstance(key, self.ValidKeywords) else self.ValidKeywords.parse(key)
         except KeyError:
-            raise KeyError("%s is not supported as an item in KustoConnectionStringBuilder" % key)
+            raise KeyError(self.kcsb_invalid_item_error % key)
 
         if value is None:
             raise TypeError("Value cannot be None.")
@@ -175,6 +187,7 @@ class KustoConnectionStringBuilder:
         """
         assert_string_is_not_empty(user_id)
         assert_string_is_not_empty(password)
+
         kcsb = cls(connection_string)
         kcsb[kcsb.ValidKeywords.aad_federated_security] = True
         kcsb[kcsb.ValidKeywords.aad_user_id] = user_id
@@ -193,6 +206,7 @@ class KustoConnectionStringBuilder:
         :param str user_token: AAD user token.
         """
         assert_string_is_not_empty(user_token)
+
         kcsb = cls(connection_string)
         kcsb[kcsb.ValidKeywords.aad_federated_security] = True
         kcsb[kcsb.ValidKeywords.user_token] = user_token
@@ -213,6 +227,7 @@ class KustoConnectionStringBuilder:
         assert_string_is_not_empty(aad_app_id)
         assert_string_is_not_empty(app_key)
         assert_string_is_not_empty(authority_id)
+
         kcsb = cls(connection_string)
         kcsb[kcsb.ValidKeywords.aad_federated_security] = True
         kcsb[kcsb.ValidKeywords.application_client_id] = aad_app_id
@@ -426,6 +441,27 @@ class KustoConnectionStringBuilder:
 
         return kcsb
 
+    @classmethod
+    def with_azure_token_credential(
+        cls,
+        connection_string: str,
+        credential: Optional[Any] = None,
+        credential_from_login_endpoint: Optional[Callable[[str], Any]] = None,
+    ) -> "KustoConnectionStringBuilder":
+        """
+        Create a KustoConnectionStringBuilder that uses an azure token credential to obtain a connection token.
+        :param connection_string: Kusto connection string should be of the format: https://<clusterName>.kusto.windows.net
+        :param credential: an optional token credential to use for authentication
+        :param credential_from_login_endpoint: an optional function that returns a token credential for the relevant kusto resource
+        """
+        kcsb = cls(connection_string)
+        kcsb[kcsb.ValidKeywords.aad_federated_security] = True
+        kcsb.is_token_credential_auth = True
+        kcsb.credential = credential
+        kcsb.credential_from_login_endpoint = credential_from_login_endpoint
+
+        return kcsb
+
     @property
     def data_source(self) -> Optional[str]:
         """The URI specifying the Kusto service endpoint.
@@ -544,6 +580,51 @@ class KustoConnectionStringBuilder:
     def domain_hint(self) -> Optional[str]:
         return self._internal_dict.get(self.ValidKeywords.domain_hint)
 
+    @property
+    def application_for_tracing(self) -> Optional[str]:
+        return self._application_for_tracing
+
+    @application_for_tracing.setter
+    def application_for_tracing(self, value: str):
+        self._application_for_tracing = value
+
+    @property
+    def user_name_for_tracing(self) -> Optional[str]:
+        return self._user_for_tracing
+
+    @user_name_for_tracing.setter
+    def user_name_for_tracing(self, value: str):
+        self._user_for_tracing = value
+
+    @property
+    def client_details(self) -> ClientDetails:
+        return ClientDetails(self.application_for_tracing, self.user_name_for_tracing)
+
+    def _set_connector_details(
+        self,
+        name: str,
+        version: str,
+        app_name: Optional[str] = None,
+        app_version: Optional[str] = None,
+        send_user: bool = False,
+        override_user: Optional[str] = None,
+        additional_fields: Optional[List[Tuple[str, str]]] = None,
+    ):
+        """
+        Sets the connector details for tracing purposes.
+        :param name:  The name of the connector
+        :param version:  The version of the connector
+        :param send_user: Whether to send the user name
+        :param override_user: Override the user name ( if send_user is True )
+        :param app_name: The name of the containing application
+        :param app_version: The version of the containing application
+        :param additional_fields: Additional fields to add to the header
+        """
+        client_details = ClientDetails.set_connector_details(name, version, app_name, app_version, send_user, override_user, additional_fields)
+
+        self.application_for_tracing = client_details.application_for_tracing
+        self.user_name_for_tracing = client_details.user_name_for_tracing
+
     def __str__(self) -> str:
         dict_copy = self._internal_dict.copy()
         for key in dict_copy:
