@@ -1,6 +1,6 @@
 # Copyright (c) Microsoft Corporation.
 # Licensed under the MIT License
-from typing import Optional, Dict, TYPE_CHECKING
+from typing import TYPE_CHECKING
 from urllib.parse import urlparse
 
 from ._token_providers import (
@@ -15,6 +15,7 @@ from ._token_providers import (
     ApplicationKeyTokenProvider,
     ApplicationCertificateTokenProvider,
     TokenConstants,
+    AzureIdentityTokenCredentialProvider,
 )
 from .exceptions import KustoAuthenticationError, KustoClientError
 
@@ -28,7 +29,11 @@ class _AadHelper:
     token_provider = None  # type: TokenProviderBase
 
     def __init__(self, kcsb: "KustoConnectionStringBuilder", is_async: bool):
-        self.kusto_uri = "{0.scheme}://{0.hostname}".format(urlparse(kcsb.data_source))
+        parsed_url = urlparse(kcsb.data_source)
+        self.kusto_uri = f"{parsed_url.scheme}://{parsed_url.hostname}"
+        if parsed_url.port is not None:
+            self.kusto_uri += f":{parsed_url.port}"
+
         self.username = None
 
         if kcsb.interactive_login:
@@ -60,7 +65,14 @@ class _AadHelper:
             self.token_provider = AzCliTokenProvider(self.kusto_uri, is_async=is_async)
         elif kcsb.token_provider or kcsb.async_token_provider:
             self.token_provider = CallbackTokenProvider(token_callback=kcsb.token_provider, async_token_callback=kcsb.async_token_provider, is_async=is_async)
-        else:
+        elif kcsb.is_token_credential_auth:
+            self.token_provider = AzureIdentityTokenCredentialProvider(
+                self.kusto_uri,
+                is_async=is_async,
+                credential=kcsb.credential,
+                credential_from_login_endpoint=kcsb.credential_from_login_endpoint,
+            )
+        else:  # TODO - next breaking change - remove this as default, make no auth the default
             self.token_provider = DeviceLoginTokenProvider(self.kusto_uri, kcsb.authority_id, is_async=is_async)
 
     def acquire_authorization_header(self):
@@ -79,6 +91,9 @@ class _AadHelper:
             kwargs["resource"] = self.kusto_uri
             raise KustoAuthenticationError(self.token_provider.name(), error, **kwargs)
 
+    def close(self):
+        self.token_provider.close()
+
 
 def _get_header_from_dict(token: dict):
     if TokenConstants.MSAL_ACCESS_TOKEN in token:
@@ -91,3 +106,18 @@ def _get_header_from_dict(token: dict):
 
 def _get_header(token_type: str, access_token: str) -> str:
     return "{0} {1}".format(token_type, access_token)
+
+
+def _is_local_address(host):
+    if host == "localhost" or host == "127.0.0.1" or host == "::1" or host == "[::1]":
+        return True
+
+    if host.startswith("127.") and 15 >= len(host) >= 9:
+        for i in range(len(host)):
+            c = host[i]
+            if c != "." and (c < "0" or c > "9"):
+                return False
+            i += 1
+        return True
+
+    return False
