@@ -5,19 +5,19 @@ import io
 import os
 import pathlib
 import platform
+import pytest
 import random
 import sys
 import time
 import uuid
+from azure.identity import DefaultAzureCredential
 from typing import Optional, ClassVar
 
-import pytest
-
 from azure.kusto.data import KustoClient, KustoConnectionStringBuilder
+from azure.kusto.data._token_providers import AsyncDefaultAzureCredential
 from azure.kusto.data.aio import KustoClient as AsyncKustoClient
 from azure.kusto.data.data_format import DataFormat, IngestionMappingKind
 from azure.kusto.data.exceptions import KustoServiceError
-
 from azure.kusto.ingest import (
     QueuedIngestClient,
     KustoStreamingIngestClient,
@@ -150,16 +150,18 @@ class TestE2E:
         return os.path.join(current_dir, *missing_path_parts)
 
     @classmethod
-    def engine_kcsb_from_env(cls) -> KustoConnectionStringBuilder:
+    def engine_kcsb_from_env(cls, is_async=False) -> KustoConnectionStringBuilder:
         if all([cls.app_id, cls.app_key, cls.auth_id]):
-            return KustoConnectionStringBuilder.with_aad_application_key_authentication(cls.engine_cs, cls.app_id, cls.app_key, cls.auth_id)
+            return KustoConnectionStringBuilder.with_azure_token_credential(
+                cls.engine_cs, credential=DefaultAzureCredential() if not is_async else AsyncDefaultAzureCredential()
+            )
         else:
             return KustoConnectionStringBuilder.with_interactive_login(cls.engine_cs)
 
     @classmethod
     def dm_kcsb_from_env(cls) -> KustoConnectionStringBuilder:
         if all([cls.app_id, cls.app_key, cls.auth_id]):
-            return KustoConnectionStringBuilder.with_aad_application_key_authentication(cls.dm_cs, cls.app_id, cls.app_key, cls.auth_id)
+            return KustoConnectionStringBuilder.with_azure_token_credential(cls.dm_cs, credential=DefaultAzureCredential())
         else:
             return KustoConnectionStringBuilder.with_interactive_login(cls.dm_cs)
 
@@ -169,8 +171,14 @@ class TestE2E:
         cls.engine_cs = os.environ.get("ENGINE_CONNECTION_STRING") or ""
         cls.dm_cs = os.environ.get("DM_CONNECTION_STRING") or cls.engine_cs.replace("//", "//ingest-")
         cls.app_id = os.environ.get("APP_ID")
+        if cls.app_id:
+            os.environ["AZURE_CLIENT_ID"] = cls.app_id
         cls.app_key = os.environ.get("APP_KEY")
+        if cls.app_key:
+            os.environ["AZURE_CLIENT_SECRET"] = cls.app_key
         cls.auth_id = os.environ.get("AUTH_ID")
+        if cls.auth_id:
+            os.environ["AZURE_TENANT_ID"] = cls.auth_id
         cls.test_db = os.environ.get("TEST_DATABASE")
         cls.test_blob = os.environ.get("TEST_BLOB")
 
@@ -228,7 +236,7 @@ class TestE2E:
 
     @classmethod
     async def get_async_client(cls) -> AsyncKustoClient:
-        return AsyncKustoClient(cls.engine_kcsb_from_env())
+        return AsyncKustoClient(cls.engine_kcsb_from_env(is_async=True))
 
     # assertions
     @classmethod
